%{
/**CFile***********************************************************************

  FileName    [input.l]

  PackageName [parser]

  Synopsis    [Lexical analyzer for the NuSMV input language]

  SeeAlso     [grammar.y]

  Author      [Marco Roveri]

  Copyright   [
  This file is part of the ``parser'' package of NuSMV version 2. 
  Copyright (C) 1998-2001 by CMU and ITC-irst. 

  NuSMV version 2 is free software; you can redistribute it and/or 
  modify it under the terms of the GNU Lesser General Public 
  License as published by the Free Software Foundation; either 
  version 2 of the License, or (at your option) any later version.

  NuSMV version 2 is distributed in the hope that it will be useful, 
  but WITHOUT ANY WARRANTY; without even the implied warranty of 
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU 
  Lesser General Public License for more details.

  You should have received a copy of the GNU Lesser General Public 
  License along with this library; if not, write to the Free Software 
  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307  USA.

  For more information of NuSMV see <http://nusmv.irst.itc.it>
  or email to <nusmv-users@irst.itc.it>.
  Please report bugs to <nusmv-users@irst.itc.it>.

  To contact the NuSMV development board, email to <nusmv@irst.itc.it>. ]

******************************************************************************/

#include <string.h> 
#include "def.h"
#include "node.h"
#include "grammar.h"

extern void yyerror(const char* s);

#define YY_DECL int yylex(YYSTYPE* yylval)

YY_DECL;

static int yycolumn = 0;

/*lint -e506 -e525 -e528 -e527 -e551 -e553 -e633 */
/*lint -e713 -e716 -e725 -e732 -e734 -e737 -e744 -e774 -e750 */
/*lint -e825 */
/*lint -esym(613,b,buf) -esym(563,find_rule) */
/*lint -esym(773,BEGIN) -esym(760,FLEX_DEBUG) */
%}
%a	15000
%o	25000

%option yylineno
%option never-interactive
%option noyywrap
%option noyymore
%option noreject
/*%option nounput*/
%pointer

%%
[ \n\t\r\f]             ;
"--"[^\n]*              ;
^"#".*\n                sscanf(yytext,"# %d",&yylineno);
"MODULE"                return(MODULE);
"process"               return(PROCESS);
"DEFINE"                return(DEFINE);
"VAR"                   return(VAR);
"IVAR"                  return(IVAR);
"INIT"                  return(INIT);
"TRANS"                 return(TRANS);
"INVAR"                 return(INVAR);
"SPEC"                  return(SPEC);
"LTLSPEC"               return(LTLSPEC);
"COMPUTE"               return(COMPUTE);
"INVARSPEC"             return(INVARSPEC);
"FAIRNESS"              return(FAIRNESS);
"JUSTICE"               return(JUSTICE);
"COMPASSION"            return(COMPASSION);
"ISA"                   return(ISA);
"ASSIGN"                return(ASSIGN);
"INPUT"                 return(INPUT);
"OUTPUT"                return(OUTPUT);
"IMPLEMENTS"            return(IMPLEMENTS);
"array"                 return(ARRAY);
"of"                    return(OF);
"boolean"               return(BOOLEAN);
"EX"                    return(EX);
"AX"                    return(AX);
"EF"                    return(EF);
"AF"                    return(AF);
"EG"                    return(EG);
"AG"                    return(AG);
"E"                     return(EE);
"F"                     return(OP_FUTURE);
"O"                     return(OP_ONCE);
"G"                     return(OP_GLOBAL);
"H"                     return(OP_HISTORICAL);
"X"                     return(OP_NEXT);
"Y"                     return(OP_PREC);
"Z"                     return(OP_NOTPRECNOT);
"A"                     return(AA);
"U"                     return(UNTIL);
"S"                     return(SINCE);
"V"                     return(RELEASES);
"T"                     return(TRIGGERED);
"BU"                    return(BUNTIL);
"EBF"                   return(EBF);
"ABF"                   return(ABF);
"EBG"                   return(EBG);
"ABG"                   return(ABG);
"MIN"                   return(MMIN);
"MAX"                   return(MMAX);
"("                     return(LP);
")"                     return(RP);
"["                     return(LB);
"]"                     return(RB);
"{"                     return(LCB);
"}"                     return(RCB);
"FALSE"                 {
                           yylval->node = node_new_bool(NODE_FALSEEXP, FALSE, NODE_NIL, NODE_NIL, yylineno, yycolumn);
                           return(FALSEEXP);
			}
"TRUE"                  {
			   yylval->node = node_new_bool(NODE_TRUEEXP, TRUE, NODE_NIL, NODE_NIL, yylineno, yycolumn);
			   return(TRUEEXP);
			}
"case"                  return(CASE);
"esac"                  return(ESAC);
":="                    return(EQDEF);
"+"                     return(PLUS);
"-"                     return(MINUS);
"*"                     return(TIMES);
"/"                     return(DIVIDE);
"mod"                   return(MOD);
"="                     return(EQUAL);
"!="                    return(NOTEQUAL);
"<="                    return(LE);
">="                    return(GE);
"<"                     return(LT);
">"                     return(GT);
"next"                  return(NEXT);
"init"                  return(SMALLINIT);
"sigma"                 return(SIGMA);
"self"                  return(SELF);
"union"                 return(UNION);
"in"                    return(SETIN);
".."                    return(TWODOTS);
"."                     return(DOT);
"->"                    return(IMPLIES);
"<->"                   return(IFF);
"|"                     return(OR);
"&"                     return(AND);
"xor"                   return(XOR);
"xnor"                  return(XNOR);
"!"                     return(NOT);
[A-Za-z_][A-Za-z0-9_\$#-]*  { 
                             yylval->node = node_new_strg(NODE_ATOM, yytext, NODE_NIL, NODE_NIL, yylineno, yycolumn);
                             return(ATOM);
                           }
[0-9]+                  {
                          yylval->node = node_new_numb(NODE_NUMBER, atoi(yytext), NODE_NIL, NODE_NIL, yylineno, yycolumn);
                          return(NUMBER);
                        }
\"[^\"]*\"              {
                          yylval->node = node_new_strg(NODE_ATOM, yytext, NODE_NIL, NODE_NIL, yylineno, yycolumn);
                          return(ATOM);
                        } /* Unused token */
","                     return(COMMA);
":"                     return(COLON);
";"                     return(SEMI);
.			fprintf(stderr, "illegal character");
%%

void yyerror(const char* s)
{
   fprintf(stderr, "*** Line %d : %s\n", yylineno, s);
#if 0
      stmt_get_filename(yystmt), stmt_get_lineno(yystmt), s);
   show_source(stderr, stmt_get_text(yystmt), yycolumn);
   abort();
#endif
}


