/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
/*                                                                           */
/*                  This file is part of the program                         */
/*        SMV2LP -- Converting SMV to Integer Linear Programs                */
/*                                                                           */
/*    Copyright (C) 2002-2005 Tobias Achterberg                              */
/*                            Thorsten Koch                                  */
/*                            Robi Malik                                     */
/*                                                                           */
/*                  2002-2005 Konrad-Zuse-Zentrum                            */
/*                            fuer Informationstechnik Berlin                */
/*                                                                           */
/*  SMV2LP is distributed under the terms of the ZIB Academic License.       */
/*                                                                           */
/*  You should have received a copy of the ZIB Academic License              */
/*  along with SMV2LP; see the file COPYING.                                 */
/*                                                                           */
/* * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * */
#ifndef __NODE_H__
#define __NODE_H__

#include "def.h"

enum node_type
{
   NODE_ERROR = 0, NODE_FALSEEXP, NODE_TRUEEXP, NODE_NUMBER, NODE_ATOM, NODE_DOT, NODE_GOTO, NODE_LET, NODE_EVAL,
   NODE_INIT, NODE_FAILNESS, NODE_TRANS, NODE_CONSTRAINT, NODE_SIMPWFF, NODE_CONTEXT, NODE_CTLWFF, NODE_LTWFF,
   NODE_EQDEF, NODE_CONS, NODE_VAR, NODE_IVAR, NODE_COLON, NODE_BOOLEAN, NODE_SCALAR, NODE_ARRAY, NODE_PROCESS,
   NODE_TWODOTS, NODE_UMINUS, NODE_MODTYPE, NODE_ASSIGN, NODE_AND, NODE_SMALLINIT, NODE_NEXT, NODE_INVAR,
   NODE_DEFINE, NODE_JUSTICE, NODE_COMPASSION, NODE_SPEC, NODE_LTLSPEC, NODE_INVARSPEC, NODE_COMPUTE, NODE_MINU,
   NODE_MAXU, NODE_SELF, NODE_IMPLIES, NODE_IFF, NODE_OR, NODE_XOR, NODE_XNOR, NODE_NOT, NODE_PLUS, NODE_MINUS,
   NODE_TIMES, NODE_IMPLEMENTS, NODE_INPUT, NODE_OUTPUT, NODE_FAIRNESS,
   NODE_DIVIDE, NODE_MOD, NODE_EQUAL, NODE_NOTEQUAL, NODE_LT, NODE_GT, NODE_LE, NODE_GE, NODE_UNION, NODE_SETIN,
   NODE_CASE, NODE_EX, NODE_AX, NODE_EF, NODE_AF, NODE_EG, NODE_AG, NODE_AU, NODE_EU, NODE_EBF,
   NODE_ABF, NODE_EBG, NODE_ABG, NODE_UNTIL, NODE_SINCE, NODE_OP_NEXT, NODE_OP_PREC, NODE_OP_NOTPRECNOT, NODE_OP_GLOBAL,
   NODE_OP_HISTORICAL, NODE_OP_FUTURE, NODE_OP_ONCE, NODE_LTLWFF, NODE_COMPWFF,
   NODE_MODULE, NODE_ABU, NODE_EBU, NODE_ISA, NODE_SIGMA,
   NODE_PROPERTY,
   NODE__MAXVAL
};

typedef struct node       Node;
typedef enum   node_type  NodeType;
typedef union  node_value NodeValue;

union node_value
{
   Bool        bool;
   Longint     numb;
   double      real;
   char*       strg;
}; 

struct node
{
   NodeType  type;
   NodeValue value;
   Node*     left;
   Node*     right;
   int       lineno;
   int       column;
};


#define NODE_NIL    ((Node*)0)
#define NODE_NOVAL  ((NodeValue)0)

extern Node* node_new(NodeType type, Node* left, Node* right, int lineno, int column);
extern Node* node_new_bool(NodeType type, Bool value, Node* left, Node* right, int lineno, int column);
extern Node* node_new_numb(NodeType type, Longint value, Node* left, Node* right, int lineno, int column);
extern Node* node_new_strg(NodeType type, const char* value, Node* left, Node* right, int lineno, int column);
extern void node_free(Node** node);

#define new_node(_node_type_,_left_,_right_) node_new(_node_type_, _left_, _right_, -1, -1) 
#define cons(_a_,_b_) node_new(NODE_CONS, _a_, _b_, -1, -1)

extern void node_display(const Node* node);
extern void tree_display(const Node* node);


#endif
